* FUNCTION nBitOp -- Perform Bitwise Operations on 2 Numbers
* 
* This function accepts 3 parameters:
* nNum1 = the primary number to be used
* nNum2 = the secondary number to be used
* cBitOp = the operation to perform.
*
* It returns the Result of the binary operation.
*
* nNum1 and nNum2 should be positive integers.  
*
* cBitOp may be "AND","NAND","OR","NOR","XOR","SHL","SHR"
*
* Note that for the Shift Left/Right Operations that the second number
* indicates the number of bits to shift the first number.
*
* 
#define dAND 0
#define dNAND 1
#define dOR 0
#define dNOR 1
#define dXOR 2

FUNCTION nBitOp
parameters nNum1, nNum2, cBitOp
private cBin1, cBin2, nTmp1, cTmp2,nResult
nResult = 0
store upper(cBitOp) to cBitOp
store "" to cBin1,cBin2

if ! lNum2Bin(nNum1,@cBin1)
  return -1
endif
if ! lNum2Bin(nNum2,@cBin2)
  return -1
endif

store max(len(cBin1),len(cBin2)) to nTmp1
store padl(cBin1,nTmp1,"0") to cBin1
store padl(cBin2,nTmp1,"0") to cBin2

do case
case cBitOp = "AND"
  cTmp2 = cBitAnd(cBin1,cBin2,dAND)
case cBitOp = "NAND"
  cTmp2 = cBitAnd(cBin1,cBin2,dNAND)
case cBitOp = "OR"
  cTmp2 = cBitOr(cBin1,cBin2,dOR)
case cBitOp = "NOR"
  cTmp2 = cBitOr(cBin1,cBin2,dNOR)
case cBitOp = "XOR"
  cTmp2 = cBitOr(cBin1,cBin2,dXOR)
case cBitOp = "SHL"
  return nNum1*2^nNum2
case cBitOp = "SHR"
  return floor(nNum1/2^nNum2)
otherwise
  return -1
endcase
if ! lBin2Num(cTmp2,@nResult)
  return -1
endif
return nResult

function lNum2Bin
parameters nNumber,cBin
private nTmp
nTmp = int(nNumber)
cBin = ""
do case
case nTmp > 0
  do while nTmp > 0
    cBin = str(mod(nTmp,2),1,0)+cBin
    nTmp = floor(nTmp / 2)
  enddo
case nTmp = 0
  store "0" to cBin
  return .t.
otherwise
  return .f.
endcase
return .t.

function lBin2Num
parameters cBin,nNum
private m.i,nLen,nDigit
nLen = len(cBin)
nNum = 0
for m.i = 1 to nLen
  nDigit = val(substr(cBin,m.i,1))
  nNum = nNum + nDigit * 2^(nLen-m.i)
endfor
return .t.

function cBitAnd
parameters cBin1,cBin2,Negate
private nLen,m.i,cResult
cResult = ""
nLen = Len(cBin1)
for m.i = 1 to nLen
  if substr(cBin1,m.i,1) = "1" and substr(cBin2,m.i,1) = "1"
    cResult = cResult + "1"
  else
    cResult = cResult + "0"
  endif
endfor
if Negate = dNAND
  cResult = chrtran(cResult,"01","10")
endif
return cResult

function cBitOr
parameters cBin1,cBin2,cOrType
private nLen,m.i,cResult,nDig1,nDig2,nSum
cResult = ""
nLen = Len(cBin1)
for m.i = 1 to nLen
  nDig1 = val(substr(cBin1,m.i,1))
  nDig2 = val(substr(cBin2,m.i,1))
  nSum = nDig1 + nDig2
  do case
  case cOrType = dOR and nSum > 0
    cResult = cResult + "1"
  case cOrType = dNOR and nSum = 0
    cResult = cResult + "1"
  case cOrType = dXOR and nSum = 1
    cResult = cResult + "1"
  otherwise
    cResult = cResult + "0"
  endcase
endfor
return cResult
